# Script 6. This script uses the master table (updated in script 4) to expand the use footprint of counties where CDL
# acreage is lower than Census acreage.

# This script has been approved for release by the U.S. Environmental Protection Agency (USEPA). Although
# the script has been subjected to rigorous review, the USEPA reserves the right to update the script as needed
# pursuant to further analysis and review. No warranty, expressed or implied, is made by the USEPA or the U.S.
# Government as to the functionality of the script and related material nor shall the fact of release constitute
# any such warranty. Furthermore, the script is released on condition that neither the USEPA nor the U.S. Government
# shall be held liable for any damages resulting from its authorized or unauthorized use.

__author__ = 'Steven Lennartz'

# Import Site Packages
import arcpy
import datetime
from arcpy import env
from arcpy.sa import *

# Set variables
# Master table from script 4
acres = r".../CDL/Analysis/NASS2012_CDL1317_180524.gdb/NASS2012_CDL1317_180524"
# Unique ID for counties, "FIPS code"
geoid = "FIPS"
# Key table for general class crop names and codes
classcode = r".../CDL/Analysis/NASS2012_CDL1317_180524.gdb/NASS2012_Genclass13_codes"
# Crop code field name. These codes are arbitrary, increments of 10, from 10 through 110.
classfield = "GenClassID"

# In-workspace is the output directory from script 3
inws = r".../CDL/Cnty_Clips/180524/1317/"
# Output directory
outws = r".../CDL/Expand/180524/1317/"
# The cultivated layer from the CDL website
maskws = r".../CDL/Cultivated_Layer/2017_Cultivated_Layer/2017_Cult_Cnty.gdb/"

start_t = datetime.datetime.now()

# Check out the ArcGIS Spatial Analyst extension license
arcpy.CheckOutExtension("Spatial")
# Environmental settings
arcpy.env.overwriteOutput = 1
arcpy.env.snapRaster = Raster(".../CDL/Cultivated_Layer/2017_Cultivated_Layer/2017_Cultivated_Layer.img")
arcpy.env.parallelProcessingFactor = "100%"

print "Begin processing at: {}".format(start_t)

# Get list of class codes
uniqueCrops = {row[0] for row in arcpy.da.SearchCursor(classcode, classfield)}

# Define function to get a value's cell count
def get_raster_count(raster_obj, value):

    counts = [row.COUNT for row in arcpy.SearchCursor(raster_obj) if row.VALUE == value]
    if not counts:
        print "No matching value"
    elif len(counts) > 1:
        print "Too many counts"
    else:
        return counts[0]
        
# Expand looping
for crop in uniqueCrops:
    crop = int(crop)
    crop = str(crop)

    # check for existing geodatabase
    if not arcpy.Exists("{}CDL{}x_cnty.gdb".format(outws, crop)):
        print "Creating CDL{}x_cnty Geodatabase".format(crop)
        arcpy.CreateFileGDB_management(outws, "CDL{}x_cnty.gdb".format(crop))

    # Return a unique set of values in the FIPS field for counties with higher NASS acreage than the CDL
    gtacres = "gtlayer"
    arcpy.MakeQueryTable_management(acres, gtacres, "USE_KEY_FIELDS", "OBJECTID", "", "NASS_{0} > CDL_{0}".format(
        crop))
    uniqueValues = {(row[0], row[1]) for row in arcpy.da.SearchCursor(gtacres, (geoid, "NASS_{}".format(crop)))}

    # Loop through counties needing expansion
    for fips, nassacres in uniqueValues:
        arcpy.env.workspace = "{}CDL{}x_cnty.gdb/".format(outws, crop)
        arcpy.env.scratchWorkspace = "{}CDL{}x_cnty.gdb/".format(outws, crop)
        fips = str(fips)
        croptime1 = datetime.datetime.now()
        print "expanding CDL{}_{}".format(crop, fips)
        # Set mask from script 5
        mask = Raster("{}Cult_2017_{}".format(maskws, fips))
        infile = Raster("{0}CDL_{1}_cnty.gdb/CDL_{1}_{2}".format(inws, crop, fips))
        # Expand flagged county by 1 pixel
        outfile = Expand(infile, 1, 1)
        # Using the environmental setting for mask does not work here. Use mask in map algebra instead.
        outfile = outfile * mask
        arcpy.BuildRasterAttributeTable_management(outfile, "Overwrite")
        count = get_raster_count(outfile, 1)
        # Iteratively compare and expand county use footprint until area >= Census, or mask area buildout.
        if count > 0:
            cdlacres = float(count * (900 / 4046.86))
            while cdlacres < nassacres:
                outfile = Expand(outfile, 1, 1)
                outfile = outfile * mask
                arcpy.BuildRasterAttributeTable_management(outfile, "Overwrite")
                count = get_raster_count(outfile, 1)
                lastacres = cdlacres
                cdlacres = float(count * (900 / 4046.86))
                if not (lastacres - cdlacres):
                    print "No change in area"
                    break
        outfile.save("{0}CDL{1}x_cnty.gdb/CDL{1}x_{2}".format(outws, crop, fips))
        print "Building pyramids...."
        arcpy.BuildPyramids_management("{0}CDL{1}x_cnty.gdb/CDL{1}x_{2}".format(outws, crop, fips))
        print "..{} expansion complete in {}".format(outfile, (datetime.datetime.now()-croptime1))
       
print "completed expansion in: {}".format(datetime.datetime.now() - start_t)
