# This script creates a list of the geodatabases (gdb) in a directory, and combines the rasters within each gdb to
# create an action area (AA). A Euclidean distance raster is then created for each AA. To create alternative AA's
# (only Ag, without developed, etc), create a gdb with the specific rasters needed and rerun script.

# This script has been approved for release by the U.S. Environmental Protection Agency (USEPA). Although
# the script has been subjected to rigorous review, the USEPA reserves the right to update the script as needed
# pursuant to further analysis and review. No warranty, expressed or implied, is made by the USEPA or the U.S.
# Government as to the functionality of the script and related material nor shall the fact of release constitute
# any such warranty. Furthermore, the script is released on condition that neither the USEPA nor the U.S. Government
# shall be held liable for any damages resulting from its authorized or unauthorized use.

__author__ = 'Steven Lennartz'

# Import Site Packages
import arcpy
from arcpy import env
from arcpy.sa import *
import datetime

# Check out the ArcGIS Spatial Analyst extension license
arcpy.CheckOutExtension("Spatial")

# Set variables.
ws = r"\path"
# The cultivated layer from the CDL website. CONUS ONLY. DO NOT APPLY FOR NL48
cult = Raster("\path")

# Environmental settings
arcpy.env.workspace = ws
arcpy.env.overwriteOutput = 1

# CONUS ONLY. DO NOT APPLY FOR NL48
arcpy.env.snapRaster = cult
arcpy.env.extent = cult

arcpy.env.parallelProcessingFactor = "100%"

# Check out the ArcGIS Spatial Analyst extension license
arcpy.CheckOutExtension("Spatial")

# Timestamp
start_t = datetime.datetime.now()
print("Starting script at: {}".format(start_t))
today = datetime.datetime.today()
date = today.strftime('%Y%m%d')

# Create a list of the rasters in the in-workspace
print("Listing Geodatabases")
gdbs = arcpy.ListWorkspaces("*", "FileGDB")

# Loop through GeoDatabases, listing rasters within each to input in cell stats to create action area
for gdb in gdbs:
    t2 = datetime.datetime.now()
    arcpy.env.workspace = gdb
    arcpy.env.scratchWorkspace = gdb
    desc = arcpy.Describe(gdb)
    # Nomenclature for Action Areas (AA). Adding the '_dev' indicates that the AA included developed and right-of-way.
    # '_Ag' for only agricultural footprints, etc.
    aa = "{}_{}".format(desc.basename, date)
    uses = arcpy.ListRasters("*")
    print("Running Cell Statistics for {} at {}".format(gdb, datetime.datetime.now()))
    out = CellStatistics(uses, "MINIMUM")
    out.save(aa)
    print("Building pyramids")
    arcpy.BuildPyramidsandStatistics_management(aa)
    print("Completed Action Area for {} in: {}".format(desc.basename, datetime.datetime.now() - t2))


print("Completed Processing of all Action Areas in: {}".format(datetime.datetime.now() - start_t))
