import datetime
import os

import arcpy
from arcpy import env
from arcpy.sa import *

# In addition to mapping the potential use site drift from these sites are mapped. The resulting layer provides the
# potential use site (VALUE 0) and the minimum distance to a use site (VALUES 30-1500). In the action area module, all
# UDLs with the associated drift are combined into a single layer. If the resulting action area has no drift values,
# i.e. only use site (VALUE 0) are present, this drift module can be executed to create a substitute drift layer.
# Similar to the action area module, the drift layer indicates the minimum distance value across all UDLs, that is
# greater than 0. A value of 0 would represent the potential use site.

# This script has been approved for release by the U.S. Environmental Protection Agency (USEPA). Although
# the script has been subjected to rigorous review, the USEPA reserves the right to update the script as needed
# pursuant to further analysis and review. No warranty, expressed or implied, is made by the USEPA or the U.S.
# Government as to the functionality of the script and related material nor shall the fact of release constitute
# any such warranty. Furthermore, the script is released on condition that neither the USEPA nor the U.S. Government
# shall be held liable for any damages resulting from its authorized or unauthorized use.

__author__ = 'Jennifer Connolly'

#    Set Variables
# In-workspace gdb contain the layers to combine
inws = r"\Path\name.gdb"
# Set out-workspace intermediate files and the final drift layer is saved here
outws = r"\Path\name.gdb"
outname = "Name_DriftLayer"

#    Environmental settings
arcpy.env.workspace = inws
arcpy.env.overwriteOutput = 1
# snap raster
arcpy.env.snapRaster = Raster(r"\Path")
arcpy.env.parallelProcessingFactor = "100%"
# Set scratch out-workspace
arcpy.env.scratchWorkspace = outws

#   Check out the ArcGIS Spatial Analyst extension license
arcpy.CheckOutExtension("Spatial")

start_t = datetime.datetime.now()
print "Started processing at: {}".format(start_t)

# List rasters in workspace
rasterList = arcpy.ListRasters()
# Loop over rasters sets potential use site (VALUE 0) to null. This values are not consider when looking for the lowest
# value that is greater than 0 across all potential use sites
for raster in rasterList:
    start_t1 = datetime.datetime.now()
    print "Processing {} at {}".format(raster, datetime.datetime.now())
    inras = Raster(raster)
    desc = arcpy.Describe(inras)
    name = desc.name
    if not arcpy.Exists(outws + os.sep + name + "_drift"):
        # #sets the use site (VALUE 0) to null; 0's will not be consider when executing the cell statistics
        print "Setting Null Values for {0} raster number {1} of {2}".format(name, (rasterList.index(raster) + 1),
                                                                            len(rasterList))
        null = SetNull(inras, inras, "VALUE = 0")
        print "Converting to integer"
        out = Int(null)
        out.save(outws + os.sep + name + "_drift")
        print "Building Pyramids....."
        arcpy.BuildPyramids_management(outws + os.sep + name + "_drift")
        print "Completed Drift Prep for {0} in: {1}".format(name, datetime.datetime.now() - start_t1)
print "Completed Euclidean Distance for all in: {0}".format(datetime.datetime.now() - start_t)

t2 = datetime.datetime.now()
# sets the workspace to the outws and gets a list of layers generated from then previous step
arcpy.env.workspace = outws
uses = arcpy.ListRasters()
# Generate drift layer by extracting the lowest value that is greater than 0.  All 0 values are now Null
print("Running Cell Statistics at {}".format(datetime.datetime.now()))
out = CellStatistics(uses, "MINIMUM")
out.save(outws + os.sep + outname)
print("Building pyramids")
arcpy.BuildPyramidsandStatistics_management(outws + os.sep + outname)
print("Completed Action Area for {} in: {}".format(desc.basename, datetime.datetime.now() - t2))
